# TODO: [ENG-7691] Switch to sidekiq once they support metrics
# FUTURE: require 'sidekiq/pro/expiry'
require 'cloud_health/collection/expiration' # And then remove this
require 'job_management'
require_relative '../job_token_limits'

logger = ImportRegistry.resolve('logger')
runtime = ImportRegistry.resolve('runtime')

redis_host = ENV['SIDEKIQ_REDIS_HOST'] || 'localhost'
redis_port = ENV['SIDEKIQ_REDIS_PORT'] || 6379

SSL_ENVS = %w(production staging)
# rediss:// is not a typo.
# Enables SSL connection within `redis` gem.
redis_uri_scheme = SSL_ENVS.include?(ENV['RACK_ENV']) ? 'rediss://' : 'redis://'
redis_url = "#{redis_uri_scheme}#{redis_host}:#{redis_port}"
logger.warn "Sidekiq redis URL: #{redis_url}, instance ID: #{ENV['SIDEKIQ_INSTANCE_ID']}"

# Make sure redis uses db 1 so it doesn't conflict with other redis usage
# https://github.com/mperham/sidekiq/issues/3543#issuecomment-325454938
# "If you are using Redis as a cache, it is recommended to put Sidekiq's data in a different database"
redis_conn = { url: redis_url, reconnect_attempts: 25, db: 1 }

::Sidekiq::Pro.dogstatsd = ->{ ImportRegistry.resolve('statsd_new') }

require 'sidekiq/middleware/server/statsd'

# Fair Queueing - register lock method and conflict strategy
# Initialize and register the token_manager here so we can access it elsewhere to get claimed token info
# Note that this requires that the redis clients be initialized first.
class ImportRegistry
  register 'job_token_mgr' do
    ::CloudHealth::JobManagement::TokenManager::RedisTokenManager.new(ImportRegistry.resolve('cache.redis_job_tokens'),
                                                                    ImportRegistry.resolve('logger'),
                                                                    GLOBAL_DEFAULT_TOKEN_CAP,
                                                                    DEFAULT_DIMENSION_CAPS,
                                                                    MEMBER_CAPS,
                                                                    ImportRegistry.resolve('statsd'))
  end
end
token_mgr = ImportRegistry.resolve('job_token_mgr')
::CloudHealth::JobManagement::Locks::TokenLock.token_manager=token_mgr

# The lock method and stategy are defined here. This step just makes them available to sidekiq
# as options, these must be set as sidekiq_options on any worker that needs to use fair queueing.
# Ex:
# sidekiq_options lock: :collector_token_lock, on_conflict: :configurable_reschedule
::SidekiqUniqueJobs.configure do |config|
    # This defines our token-based locking strategy
    config.add_lock :derived_processor_token_lock, Gdas::JobMgmt::DerivedProcessorTokenLock
    config.enabled = true
    # This tells sidekiq what to do if we fail to get a lock
    config.add_strategy :configurable_reschedule, Gdas::JobMgmt::ConfigurableReschedule
end

::Sidekiq.configure_server do |config|
  config.redis = redis_conn
  config.logger = logger

  config.client_middleware do |chain|
    chain.add ::CloudHealth::JobManagement::Sidekiq::Middleware::Client::UniqueJobIdMiddleware
    chain.add SidekiqUniqueJobs::Middleware::Client
  end

  config.server_middleware do |chain|
    chain.add ::Sidekiq::Middleware::Server::Statsd
    chain.add SidekiqUniqueJobs::Middleware::Server
  end

  SidekiqUniqueJobs::Server.configure(config)

  # https://github.com/mperham/sidekiq/wiki/Scheduled-Jobs#checking-for-new-jobs
  # 5s is overkill for our fanout jobs
  config.average_scheduled_poll_interval = 15

  # Turn on reliability: https://github.com/mperham/sidekiq/wiki/Pro-Reliability-Server
  config.super_fetch!
end

::Sidekiq.configure_client do |config|
  config.redis = redis_conn
  config.logger = logger

  config.client_middleware do |chain|
    chain.add ::CloudHealth::JobManagement::Sidekiq::Middleware::Client::UniqueJobIdMiddleware
    chain.add SidekiqUniqueJobs::Middleware::Client
  end
end

# https://github.com/mperham/sidekiq/wiki/Pro-Reliability-Client
# This should not go in a Sidekiq.configure_{client,server} block.
::Sidekiq::Client.reliable_push! unless runtime.test?
